"""update auth code model, add action

Revision ID: fafdb22e5a16
Revises: 491aaa1a58f6
Create Date: 2026-01-23 11:57:52.119578

"""
from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa
import advanced_alchemy


# revision identifiers, used by Alembic.
revision: str = 'fafdb22e5a16'
down_revision: Union[str, Sequence[str], None] = '491aaa1a58f6'
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    """Upgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('auth_codes', sa.Column('action', sa.String(), nullable=False))
    # ### end Alembic commands ###


def downgrade() -> None:
    """Downgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('auth_codes', 'action')
    # ### end Alembic commands ###
